#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs = tslib_1.__importStar(require("fs"));
var path = tslib_1.__importStar(require("path"));
var commander_1 = tslib_1.__importDefault(require("commander")); // @fix no import * https://github.com/microsoft/tslib/issues/58
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var index_1 = tslib_1.__importDefault(require("./index"));
var CD = process.cwd();
commander_1.default
    .version(require('../package.json').version)
    .option('-c, --config [path]', 'config file', 'json2service.json')
    .option('--clear', 'rm typescript service before gen', false)
    .option('--quiet', 'auto merge without popup', false)
    .option('--apis [boolean]', '是否生成 apis，如果未指定 models 且未指定 typeScriptDataFile 则会生成 apis', undefined)
    .option('--models [boolean]', '是否生成 models，如果未指定 apis 且未指定 typeScriptDataFile 则会生成 models', undefined)
    .option('--typeScriptDataFile [string]', '是否仅生成 TypeScript Data，可用于完全自定义生成逻辑', undefined)
    .parse(process.argv);
var Config = commander_1.default.config;
var ConfigFile = path.join(CD, Config);
if (!fs.existsSync(ConfigFile)) {
    console.log(chalk_1.default.red("[ERROR]: " + Config + " not found in " + CD));
}
else {
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    var config = require(ConfigFile);
    var clear = commander_1.default.clear, quiet = commander_1.default.quiet, typeScriptDataFile = commander_1.default.typeScriptDataFile, apis = commander_1.default.apis, models = commander_1.default.models;
    index_1.default(config, { clear: clear, quiet: quiet, typeScriptDataFile: typeScriptDataFile, apis: apis, models: models })
        .then(function () {
        process.exit(0);
    })
        .catch(function (e) {
        console.log(chalk_1.default.red(e));
        process.exit(1);
    });
}
//# sourceMappingURL=cli.js.map