"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path = tslib_1.__importStar(require("path"));
var fs = tslib_1.__importStar(require("fs-extra"));
var ajv_1 = tslib_1.__importDefault(require("ajv"));
var consts_1 = require("../consts");
var init_1 = require("../init");
var utils_1 = require("./utils");
var formatSchema = function (schema) {
    return JSON.parse(JSON.stringify(schema).replace(/"#\/definitions/g, '"definitions#/definitions'));
};
/** 将 swagger 转成 ajv */
exports.getAJV = (function () {
    var inst;
    var lastSwagger;
    return function (swagger) {
        if (lastSwagger === swagger && inst) {
            return inst;
        }
        lastSwagger = swagger;
        inst = new ajv_1.default();
        inst.addFormat('int32', function (d) {
            return !!d.match(/^[0-9-]+$/);
        });
        var definitions = getDefinitions(swagger);
        if (definitions) {
            inst.addSchema({
                $id: 'definitions',
                definitions: formatSchema(definitions)
            });
        }
        return inst;
    };
})();
/** 根据 base path 获取 swagger 文件 */
function getSwagger(basePath) {
    var swaggerPath = path.join(init_1.SmTmpDir, init_1.basePathToFileName((basePath || init_1.DefaultBasePath) + ".json"));
    if (fs.existsSync(swaggerPath)) {
        var swagger = require(swaggerPath);
        var state = fs.statSync(swaggerPath);
        if (swagger.__mtime && swagger.__mtime !== state.mtime) {
            delete require.cache[require.resolve(swaggerPath)];
            swagger = require(swaggerPath);
        }
        swagger.__mtime = state.mtime;
        return swagger;
    }
    else {
        throw Error('物料文件不存在，请重新生成');
    }
}
exports.getSwagger = getSwagger;
/** 获取 swagger definitions 字段 */
function getDefinitions(swagger) {
    var definitions = swagger.definitions;
    return definitions;
}
exports.getDefinitions = getDefinitions;
/** 根据 path 从 swagger paths 内获取接口定义 */
function getPath(swagger, path) {
    var paths = swagger.paths;
    var matched = paths[path];
    if (!matched) {
        // path 内包含变量
        Object.keys(paths).find(function (p) {
            if (p.indexOf('{') !== -1 && path.match(utils_1.pathToReg(p))) {
                matched = paths[p];
                return true;
            }
            return false;
        });
    }
    if (!matched) {
        throw Error("\u63A5\u53E3 \"" + path + "\" \u4E0D\u5B58\u5728");
    }
    return matched;
}
exports.getPath = getPath;
/** 将 swagger parameters 转换成 schema */
function getParamSchema(parameters) {
    var paramsSchema = {
        type: 'object',
        properties: {}
    };
    return parameters.reduce(function (parentSchema, param) {
        var _a = param.in, paramType = _a === void 0 ? '' : _a, schema = param.schema, _b = param.name, name = _b === void 0 ? '' : _b, type = param.type, required = param.required, description = param.description, format = param.format;
        if (parentSchema.properties) {
            var cur = parentSchema.properties[paramType];
            if (schema) {
                parentSchema.properties[paramType] = formatSchema(schema);
            }
            else {
                // 非 post json 转换成 schema
                if (!cur) {
                    cur = parentSchema.properties[paramType] = {
                        type: 'object'
                    };
                }
                if (!cur.properties) {
                    cur.properties = {};
                }
                cur.properties[name] = {
                    type: type,
                    description: description,
                    format: format
                };
                // object 通过 required 字段来约束必选字段
                if (required) {
                    cur.required = cur.required || [];
                    cur.required.push(name);
                }
            }
        }
        return parentSchema;
    }, paramsSchema);
}
exports.getParamSchema = getParamSchema;
exports.validatorFactory = function (ajv) { return function (shema, data) {
    return ajv.validate(shema, data) ? '' : ajv.errorsText(ajv.errors);
}; };
/** @tkit/ajax 规范的数据响应的校验 */
exports.defaultValidator = function (_a, _b) {
    var code = _a.code, message = _a.message, result = _a.result;
    var loadSwagger = _b.loadSwagger, onValidate = _b.onValidate, formatBodyBeforeValidate = _b.formatBodyBeforeValidate;
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var _c, u, send, _d, body, status, url, swagger, _e, basePath, path_1, baseInfo, validator, method, matchMethod, parameters, responses, error, schema, error;
        var _f, _g, _h, _j, _k;
        return tslib_1.__generator(this, function (_l) {
            _c = result.req.url, u = _c === void 0 ? '' : _c, send = result.send, _d = result.receive, body = _d.body, status = _d.status;
            try {
                url = u.split('?')[0];
                if (code) {
                    throw Error("\u63A5\u53E3 " + url + " \u521D\u59CB\u5316\u51FA\u9519: " + message);
                }
                else {
                    swagger = loadSwagger(url);
                    _e = swagger.basePath, basePath = _e === void 0 ? '' : _e;
                    path_1 = url.replace(new RegExp("^" + basePath), '');
                    baseInfo = "" + path_1 + (url.indexOf(path_1) === -1 ? " / " + url : '');
                    validator = exports.validatorFactory(exports.getAJV(swagger));
                    method = (send ? send.method : 'GET').toLocaleLowerCase();
                    matchMethod = getPath(swagger, path_1)[method];
                    if (!matchMethod) {
                        throw Error("\u63A5\u53E3 " + baseInfo + " \u4E0D\u652F\u6301 " + method);
                    }
                    parameters = matchMethod.parameters, responses = matchMethod.responses;
                    if (parameters && parameters.length && send) {
                        error = validator(getParamSchema(parameters), send);
                        if (error) {
                            (_f = onValidate) === null || _f === void 0 ? void 0 : _f({
                                code: consts_1.ValidateErrorCode.ParamsNotMatched,
                                message: "\u63A5\u53E3 " + baseInfo + " \u53C2\u6570\u4E0D\u7B26\u5408\u7EA6\u5B9A: " + error,
                                result: result
                            });
                        }
                        else {
                            (_g = onValidate) === null || _g === void 0 ? void 0 : _g({
                                code: 0,
                                message: "\u63A5\u53E3 " + baseInfo + " \u53C2\u6570\u7B26\u5408\u7EA6\u5B9A",
                                result: result
                            });
                        }
                    }
                    schema = responses[status].schema;
                    if (schema) {
                        error = validator(formatSchema(schema), formatBodyBeforeValidate ? formatBodyBeforeValidate(body) : body);
                        if (error) {
                            (_h = onValidate) === null || _h === void 0 ? void 0 : _h({
                                code: consts_1.ValidateErrorCode.ResponseNotMatched,
                                message: "\u63A5\u53E3 " + baseInfo + " \u6570\u636E\u8FD4\u56DE\u4E0D\u7B26\u5408\u7EA6\u5B9A: " + error,
                                result: result
                            });
                        }
                        else {
                            (_j = onValidate) === null || _j === void 0 ? void 0 : _j({
                                code: 0,
                                message: "\u63A5\u53E3 " + baseInfo + " \u6570\u636E\u8FD4\u56DE\u7B26\u5408\u7EA6\u5B9A",
                                result: result
                            });
                        }
                    }
                    else if (body) {
                        throw Error("\u63A5\u53E3 " + baseInfo + " \u4E0D\u5E94\u8BE5\u6709\u6570\u636E\u8FD4\u56DE");
                    }
                }
            }
            catch (e) {
                (_k = onValidate) === null || _k === void 0 ? void 0 : _k({
                    code: consts_1.ValidateErrorCode.Weird,
                    message: e.message
                });
            }
            return [2 /*return*/];
        });
    });
};
//# sourceMappingURL=validator.js.map